'use client';
import React, { useState, useEffect, useRef } from 'react';
import CryptoJS from 'crypto-js';

const Counter = () => {
  const [hours, setHours] = useState(0);
  const [minutes, setMinutes] = useState(0);
  const [seconds, setSeconds] = useState(0);
  const [isCounting, setIsCounting] = useState(false);
  const workerRef = useRef<Worker | null>(null);

   // Reiniciar el contador al inicio de cada día
   const resetCounterDaily = () => {
    const currentDate = new Date().toLocaleDateString();
    const storedDate = localStorage.getItem('counterDate');
    const entryTimeStr = localStorage.getItem('entryTime');

    // Si existe entryTime, calcular la diferencia y usarla para inicializar el contador
    if (entryTimeStr) {
      const entryTime = new Date(entryTimeStr);
      const now = new Date();
      let diffMs = now.getTime() - entryTime.getTime();
      if (diffMs < 0) diffMs = 0; // Evitar negativos
      const diffSeconds = Math.floor(diffMs / 1000);
      const hoursFromEntry = Math.floor(diffSeconds / 3600);
      const minutesFromEntry = Math.floor((diffSeconds % 3600) / 60);
      const secondsFromEntry = diffSeconds % 60;

      setHours(hoursFromEntry);
      setMinutes(minutesFromEntry);
      setSeconds(secondsFromEntry);
      setIsCounting(true);
      // Guardar en localStorage cifrado como hasta ahora
      const data = JSON.stringify({ hours: hoursFromEntry, minutes: minutesFromEntry, seconds: secondsFromEntry });
      const encryptedData = CryptoJS.AES.encrypt(data, process.env.SECRET_CRYPTO_JS || '').toString();
      localStorage.setItem('counter', encryptedData);
      localStorage.setItem('counterDate', currentDate);
      // Iniciar el worker con estos valores
      workerRef.current?.postMessage({
        type: 'START_TIMER',
        payload: { hours: hoursFromEntry, minutes: minutesFromEntry, seconds: secondsFromEntry }
      });
      return;
    }

    // Si no existe entryTime, comportamiento previo
    if (currentDate !== storedDate) {
      setHours(0);
      setMinutes(0);
      setSeconds(0);
      setIsCounting(true);
      localStorage.setItem('counterDate', currentDate);
      localStorage.setItem('counter', '');
      // Start the worker with initial values
      workerRef.current?.postMessage({
        type: 'START_TIMER',
        payload: { hours: 0, minutes: 0, seconds: 0 }
      });
    } else {
      // Cargar el contador del almacenamiento local
      const encryptedDataGet: string = localStorage.getItem('counter') || '';
      if (encryptedDataGet) {
        const bytes = CryptoJS.AES.decrypt(encryptedDataGet, process.env.SECRET_CRYPTO_JS || '');
        const decryptedData = bytes.toString(CryptoJS.enc.Utf8);
        if (decryptedData) {
          const { hours: savedHours, minutes: savedMinutes, seconds: savedSeconds } = JSON.parse(decryptedData);
          setHours(savedHours);
          setMinutes(savedMinutes);
          setSeconds(savedSeconds);
          setIsCounting(true);
          // Start the worker with saved values
          workerRef.current?.postMessage({
            type: 'START_TIMER',
            payload: { hours: savedHours, minutes: savedMinutes, seconds: savedSeconds }
          });
        }
      } else {
        setIsCounting(true);
        // Start the worker with initial values
        workerRef.current?.postMessage({
          type: 'START_TIMER',
          payload: { hours: 0, minutes: 0, seconds: 0 }
        });
      }
    }
  };
  
  useEffect(() => {
    // Create the worker
    if (typeof window !== 'undefined') {
      // We need to create the worker only on the client side
      workerRef.current = new Worker(
        new URL('./counterWorker.ts', import.meta.url)
      );

      // Listen for messages from the worker
      workerRef.current.addEventListener('message', (event) => {
        const { type, payload } = event.data;
        if (type === 'TICK') {
          setHours(payload.hours);
          setMinutes(payload.minutes);
          setSeconds(payload.seconds);
        }
      });
    }

   

    resetCounterDaily();

    // Cleanup function to terminate the worker when component unmounts
    return () => {
      if (workerRef.current) {
        workerRef.current.postMessage({ type: 'STOP_TIMER' });
        workerRef.current.terminate();
      }
    };
  }, []);

  useEffect(() => {
    if (isCounting) {
      const data = JSON.stringify({ hours, minutes, seconds });
      const encryptedData = CryptoJS.AES.encrypt(data, process.env.SECRET_CRYPTO_JS || '').toString();
      localStorage.setItem('counter', encryptedData);
    }
  }, [hours, minutes, seconds, isCounting]);

  useEffect(() => {
    const handleVisibilityChange = () => {
      if (document.visibilityState === 'visible') {
        resetCounterDaily();
      }
    };
  
    document.addEventListener('visibilitychange', handleVisibilityChange);
  
    return () => {
      document.removeEventListener('visibilitychange', handleVisibilityChange);
    };
  }, []);

  return (
    <div className='m-2 border-2 md:border-gray-400 p-1 rounded-md w-24 flex items-center justify-center'>
      <p className="text-lg font-bold family-sans-serif">{`${hours.toString().padStart(2, '0')}:${minutes.toString().padStart(2, '0')}:${seconds.toString().padStart(2, '0')}`}</p>
    </div>
  );
};

export default Counter;
